@@ -10,7 +10,7 @@ import yaml
 from click import UsageError, Option
 from click.decorators import FC
 
-from .. import DataConnectClient, WesClient, CollectionsClient, PublisherClient
+from .. import DataConnectClientV2, WesClient, CollectionsClient, PublisherClient
 from ..auth import (
     PersonalAccessTokenAuth,
     DeviceCodeAuth,
@@ -18,6 +18,7 @@ from ..auth import (
     OAuthClientParams,
 )
 from ..client import BaseServiceClient
+from ..client.dataconnect_client_slim import DataConnectClientV2
 from ..exceptions import NoConfigException, InvalidConfigTypeException, ConfigException
 from ..constants import (
     ACCEPTED_CONFIG_KEYS,
@@ -35,7 +36,7 @@ def get_client(ctx: click.Context):
 
 def get_service_config_key(service: Any) -> str:
     """Get the configuration base key for a :class:`ServiceType`"""
-    if isinstance(service, DataConnectClient):
+    if isinstance(service, DataConnectClientV2):
         return "data_connect"
     elif isinstance(service, CollectionsClient):
         return "collections"
@@ -341,7 +342,7 @@ def create_publisher_client_from_config(ctx: click.Context) -> None:
     publisher_client = PublisherClient()
 
     publisher_client.dataconnect = create_service_client_from_config(
-        ctx, "data_connect", DataConnectClient
+        ctx, "data_connect", DataConnectClientV2
     )
     publisher_client.collections = create_service_client_from_config(
         ctx, "collections", CollectionsClient
@@ -459,9 +460,7 @@ def format_var_path(var_path: list) -> None:
             var_path[i] = format_url_for_config(var_path[i])
 
 
-def format_query_result_as_csv(
-    query_results: List[dict], include_headers: bool = True
-) -> str:
+def format_query_result_as_csv(query_results: List[dict], include_headers: bool = True) -> str:
     output = io.StringIO()
     writer = csv.writer(output)
 
