@@ -1,5 +1,5 @@
 from .files_client import FilesClient
-from .dataconnect_client import DataConnectClient
+from .dataconnect_client_slim import DataConnectClientV2
 from .collections_client import CollectionsClient
 from .wes_client import WesClient
 from .publisher_client import PublisherClient
