@@ -1,20 +1,14 @@
+import os
+import re
+import threading
 from enum import Enum
-from urllib.parse import urlparse, urlunparse
-from requests import Request
+from typing import Optional, Union, List, AnyStr
+from urllib.parse import urlparse, urljoin
 
 import click
 import urllib3
-import threading
-from typing import Optional, Union, List, AnyStr
 
 from .base_client import BaseServiceClient
-from dnastack.constants import *
-from requests.exceptions import HTTPError
-import re
-from uuid import UUID
-import pandas as pd
-from search_python_client.search import DrsClient
-
 from ..auth import OAuthTokenAuth
 from ..constants import DEFAULT_SERVICE_REGISTRY
 from ..exceptions import DRSDownloadException, DRSException
@@ -28,34 +22,22 @@ class DRSObject:
     :raises ValueError if url is not a valid DRS url
     """
 
+    __RE_VALID_DRS_OBJECT_ID = re.compile(r'^[^/#?]+$')
+
     def __init__(self, url: str):
-
-        self.url = url
-
-        if not DRSObject.is_drs_url(url):
+        try:
+            DRSObject.assert_valid_drs_url(url)
+        except AssertionError:
             raise ValueError("The provided url is not a valid DRS url")
 
-        self.object_id = DRSObject._get_object_id_from_url(url)
-        self.drs_server = self._get_drs_server_from_url(url)
+        self.__url = url
 
-    @staticmethod
-    def _get_drs_server_from_url(url: str) -> str:
-        """
-        Return the HTTPS server associated with the DRS url
+    @property
+    def url(self):
+        return self.__url
 
-        :param url: A drs url
-        :return: The associated HTTPS server url
-        """
-        parsed_url = urlparse(url)
-        path_prefix = "/".join(parsed_url.path.split("/")[:-1])
-        return str(
-            urlunparse(
-                ("https", parsed_url.netloc, path_prefix + "/ga4gh/drs/v1/", "", "", "")
-            )
-        )
-
-    @staticmethod
-    def _get_object_id_from_url(url: str) -> str:
+    @property
+    def object_id(self) -> str:
         """
         Return the object ID from a drs url
 
@@ -63,25 +45,27 @@ class DRSObject:
         :return: The object ID extracted from the URL
         :raises: ValueError if there isn't a valid DRS Object ID
         """
-        parsed_url = urlparse(url)
-        url_object_id = parsed_url.path.split("/")[-1]
-        try:
-            object_id = UUID(url_object_id, version=4)
-        except ValueError:
-            raise ValueError(f"{url_object_id} is not a valid UUID")
+        parsed_url = urlparse(self.url)
+        return parsed_url.path.split("/")[-1]
 
-        return str(object_id)
+    @property
+    def drs_server_url(self) -> str:
+        """
+        Return the HTTPS server associated with the DRS url
+
+        :param url: A drs url
+        :return: The associated HTTPS server url
+        """
+        parsed_url = urlparse(self.url)
+        return urljoin(f'https://{parsed_url.netloc}{"/".join(parsed_url.path.split("/")[:-1])}', 'ga4gh/drs/v1/')
 
     @staticmethod
-    def is_drs_url(url: str) -> bool:
+    def assert_valid_drs_url(url: str):
         """Returns true if url is a valid DRS url"""
         parsed_url = urlparse(url)
-        return parsed_url.scheme == "drs"
-
-
-def get_host(url: str) -> str:
-    parsed_url = urlparse(url)
-    return parsed_url.netloc
+        assert parsed_url.scheme == r'drs', f'The scheme of the given URL ({url}) is invalid.'
+        assert len(parsed_url.path) > 2 and parsed_url.path.startswith(r'/'), f'The ID is not specified in the URL ({url}).'
+        assert DRSObject.__RE_VALID_DRS_OBJECT_ID.search(parsed_url.path[1:]), f'The format of the ID ({parsed_url.path[1:]}) is not valid.'
 
 
 def handle_file_response(download_file: str, data: Union[str, bytes]) -> str:
@@ -125,20 +109,12 @@ class FilesClient(BaseServiceClient):
     ):
 
         # A lock to prevent race conditions on exit_codes objects
-        self.output_lock = threading.Lock()
+        self.__output_lock = threading.Lock()
         # lock to prevent race conditions for file output
-        self.exit_code_lock = threading.Lock()
+        self.__exit_code_lock = threading.Lock()
 
         super().__init__(auth=auth, registry_url=registry_url)
 
-    def get_drs_client(self, drs_server: str) -> DrsClient:
-
-        if self.auth:
-            req = Request(url=drs_server)
-            return DrsClient(drs_server, wallet=self.auth.get_access_token(req))
-        else:
-            return DrsClient(drs_server)
-
     def exit_download(
         self,
         url: str,
@@ -155,16 +131,15 @@ class FilesClient(BaseServiceClient):
         :param exit_codes: A shared dict for all reports used by download_files
         """
         if exit_codes is not None:
-            self.exit_code_lock.acquire()
-            exit_codes[status][url] = message
-            self.exit_code_lock.release()
+            with self.__exit_code_lock:
+                exit_codes[status][url] = message
 
     def download_file(
         self,
         url: str,
         output_dir: str,
         display_progress_bar: bool = False,
-        out: Optional[list] = None,
+        output_buffer_list: Optional[list] = None,
         exit_codes: Optional[dict] = None,
     ) -> None:
         """
@@ -173,15 +148,10 @@ class FilesClient(BaseServiceClient):
         :param url: The DRS resource url to download
         :param output_dir: The directory to download output to.
         :param display_progress_bar: Display a progress bar for the downloads to standard output
-        :param out: If specified, output downloaded data to the list specified in the argument
+        :param output_buffer_list: If specified, output downloaded data to the list specified in the argument
         :param exit_codes: A shared dictionary of the exit statuses and messages
         :return:
         """
-
-        http = urllib3.PoolManager()
-        chunk_size = 1024
-        download_url = None
-
         try:
             drs_object = DRSObject(url)
         except ValueError as v:
@@ -193,29 +163,30 @@ class FilesClient(BaseServiceClient):
             )
             return
 
-        drs_client = self.get_drs_client(drs_object.drs_server)
+        object_info_response = self.client.get(urljoin(drs_object.drs_server_url, f'objects/{drs_object.object_id}'))
 
-        try:
-            object_info = drs_client.get_object_info(drs_object.object_id)
-        except HTTPError as e:
-            if e.response.status_code == 404:
+        object_info_status_code = object_info_response.status_code
+        if object_info_status_code != 200:
+            if object_info_status_code == 404:
                 error_msg = f"DRS object at url [{url}] does not exist"
-            elif e.response.status_code == 403:
+            elif object_info_status_code == 403:
                 error_msg = "Access Denied"
             else:
                 error_msg = "There was an error getting object info from the DRS Client"
-            http.clear()
             self.exit_download(url, DownloadStatus.FAIL, error_msg, exit_codes)
             return
 
-        if "access_methods" in object_info.keys():
-            access_methods = object_info["access_methods"][0]
-            for access_method in [am for am in access_methods if am["type"] == "https"]:
+        object_info = object_info_response.json()
+        download_url: Optional[str] = None
+
+        if "access_methods" in object_info and object_info['access_methods']:
+            for access_method in object_info["access_methods"]:
+                if access_method["type"] != "https":
+                    continue
                 # try to use the access_id to get the download url
                 if "access_id" in access_method.keys():
-                    object_access = drs_client.get_object_access(
-                        drs_object.object_id, access_method["access_id"]
-                    )
+                    object_access_response = self.client.get(urljoin(drs_object.drs_server_url, f'objects/{drs_object.object_id}/access/{access_method["access_id"]}'))
+                    object_access = object_access_response.json()
                     download_url = object_access["url"][0]
                     break
                 # if we have a direct access_url for the access_method, use that
@@ -225,7 +196,6 @@ class FilesClient(BaseServiceClient):
 
             if not download_url:
                 # we couldn't find a download url, exit unsuccessful
-                http.clear()
                 self.exit_download(
                     url,
                     DownloadStatus.FAIL,
@@ -235,10 +205,13 @@ class FilesClient(BaseServiceClient):
         else:
             return  # next page token, just return
 
+        http_connection_pool = urllib3.PoolManager()
+        chunk_size = 1024
+
         try:
-            download_stream = http.request("GET", download_url, preload_content=False)
-        except HTTPError as e:
-            http.clear()
+            download_stream = http_connection_pool.request("GET", download_url, preload_content=False)
+        except Exception as e:
+            http_connection_pool.clear()
             self.exit_download(
                 url,
                 DownloadStatus.FAIL,
@@ -249,30 +222,24 @@ class FilesClient(BaseServiceClient):
 
         download_filename = get_filename_from_url(download_url)
 
-        if out is not None:
+        if output_buffer_list is not None:
             data = handle_file_response(download_filename, download_stream.read())
-            self.output_lock.acquire()
-            out.append(file_to_dataframe(download_filename, data))
-            self.output_lock.release()
-
+            with self.__output_lock:
+                output_buffer_list.append(file_to_dataframe(download_filename, data))
         else:
             with open(f"{output_dir}/{download_filename}", "wb+") as dest:
                 stream_size = int(download_stream.headers["Content-Length"])
                 file_stream = download_stream.stream(chunk_size)
                 if display_progress_bar:
-                    click.echo(
-                        f"Downloading {url} into {output_dir}/{download_filename}"
-                    )
-                    with click.progressbar(
-                        length=stream_size, color=True
-                    ) as download_progress:
+                    click.echo(f"Downloading {url} into {output_dir}/{download_filename}...")
+                    with click.progressbar(length=stream_size, color=True) as download_progress:
                         for chunk in file_stream:
                             dest.write(chunk)
-                            download_progress.update(chunk_size)
+                            download_progress.update(len(chunk))
                 else:
                     for chunk in file_stream:
                         dest.write(chunk)
-        http.clear()
+        http_connection_pool.clear()
         self.exit_download(
             url, DownloadStatus.SUCCESS, "Download Successful", exit_codes
         )
@@ -305,7 +272,7 @@ class FilesClient(BaseServiceClient):
                     url,
                     output_dir,
                     display_progress_bar=display_progress_bar,
-                    out=out,
+                    output_buffer_list=out,
                     exit_codes=exit_codes,
                 ),
                 name=url,
